% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BayCount_MCMC.R
\name{BayCount_MCMC}
\alias{BayCount_MCMC}
\title{The Gibbs sampler for BayCount with fixed K}
\usage{
BayCount_MCMC(Y, Init, hyper, B, nmc, K, Print_Iter = FALSE,
  lab_switch.check = FALSE)
}
\arguments{
\item{Y}{observed count matrix}

\item{Init}{an Init object used for initializing the Markov chain}

\item{hyper}{the hyperparameters for the MCMC}

\item{B}{number of burn-in iterations}

\item{nmc}{number of post-burn-in iterations}

\item{K}{number of subclones}

\item{Print_Iter}{logical variable indicating whether or not to print the number of current iteration}

\item{lab_switch.check}{logical variable indicating whether label switching phenomenon is checked}
}
\value{
mcmc an mcmc object for BayCount
}
\description{
MCMC main function for BayCount with fixed number of subclones
}
\examples{
# Let us consider a medium scale synthetic example.
# In particular, we consider the synthetic_data dataset
# This dataset is used in Section 3.1 in the manuscript
# loading required packages
library(Rcpp)
library(combinat)
library(RColorBrewer)
library(gplots)
# Let us initialize the hyperparameters
hyper=NULL
hyper$eta = 0.1
hyper$a0 = 0.01
hyper$b0 = 0.01
hyper$e0 = 1
hyper$f0 = 1
hyper$g0 = 1
hyper$h0 = 1
hyper$u0 = hyper$v0 = 100
B = 1000
nmc = 1000
K_range = 1:10
sample_ind = (B + 1):(B + nmc)

###############################################################
# Run the BayCount model with known K                         #
###############################################################
attach(synthetic_data)
G = dim(Y)[1]
S = dim(Y)[2]
K = dim(Phi)[2]
Init = list(Phi = matrix(1/G, nrow = G, ncol = K), Theta = matrix(15, nrow = K, ncol = S))
BayCount_inference = BayCount_MCMC(Y, Init, hyper, B, nmc, K, Print_Iter = TRUE)
BayCount_inference = BayCount_match_lable(BayCount_inference, simu_truth = synthetic_data)
Phi_hat = apply(BayCount_inference$Phi[, , sample_ind], c(1, 2), mean)
Theta_hat_norm = apply(BayCount_inference$Theta_norm[, , sample_ind], c(1, 2), mean)

###############################################################
# Compare estimated subclonal proportions with the ground truth
###############################################################
pdf("subclones_Proportions_medium_scale.pdf", width = 10, height = 12)
par(mfrow=c(K, 1))
for (k in 1:K){
  theta_k_estimate = Theta_hat_norm[k, ]
  theta_k_CI_upper = apply(BayCount_inference$Theta_norm[k, , sample_ind], 1, quantile, 0.975)
  theta_k_CI_lower = apply(BayCount_inference$Theta_norm[k, , sample_ind], 1, quantile, 0.025)
  plot(1:S, type = "n", xlab = "Samples", ylab = "Proportions",
       ylim = c(min(theta_k_CI_lower), max(theta_k_CI_upper) + 0.4),
       main = paste("Subclone ",k, " Proportion", sep = ""), cex.main = 2, cex.axis = 1.5, cex.lab = 1.5)
  grid(nx = 10, ny = 5, lwd = 1)
  polygon(c(1:S, rev(1:S)), c(theta_k_CI_lower, rev(theta_k_CI_upper)),col = "grey80", border = T)
  points(1:S, theta_k_estimate, type="b", pch = 1, lwd = 3, col = "green", lty = 1)
  points(1:S, synthetic_data$Theta_norm[k, ], type = "b",pch = 2, col = "red", lwd = 3, lty = 1)
  legend("topleft", legend = c("Estimates","True Values"), lwd = c(3, 3),
         pch = c(1, 2), col = c("green","red"), cex = 1.5, bty = "n")
}
dev.off()

###############################################################
# Heatmaps of gene expression data                            #
###############################################################
sd_gene = apply(synthetic_data$Phi_norm, 1, sd)
tau_sd = 1/G
diff_exp_gene = which(sd_gene > tau_sd)
my_palette = brewer.pal(9, "Blues")
pdf("Heat_map_gene_exp_true.pdf", width = 5, height = 9)
heatmap.2(synthetic_data$Phi[diff_exp_gene, ], trace = "none", Rowv = NULL, Colv = NULL,
          dendrogram = "none", col = my_palette, key = TRUE, margins = c(3, 0),
          key.par = list(mar = c(2, 0,3, 0)), denscol = "black", key.xlab = "", key.ylab = "",
          key.title = "", labRow = "", cexCol = 4, lhei = c(0.8, 5), lwid = c(0.2, 1.2, 0.2),
          lmat=rbind(c(5, 4, 2), c(6, 1, 3)))
dev.off()
pdf("Heat_map_gene_exp_estimate.pdf", width = 5, height = 9)
heatmap.2(Phi_hat[diff_exp_gene, ], trace = "none", Rowv = NULL, Colv = NULL,
          dendrogram = "none", col = my_palette, key = TRUE, margins = c(3, 0),
          key.par = list(mar = c(2, 0,3, 0)), denscol = "black", key.xlab = "", key.ylab = "",
          key.title = "", labRow = "", cexCol = 4, lhei = c(0.8, 5), lwid = c(0.2, 1.2, 0.2),
          lmat=rbind(c(5, 4, 2), c(6, 1, 3)))
dev.off()
}
